@@ -1,13 +1,8 @@
 import logging
 import os
-from typing import TYPE_CHECKING, Set
+from typing import TYPE_CHECKING, Dict, List, Set
 
-from dvc.exceptions import (
-    CheckoutError,
-    CheckoutErrorSuggestGit,
-    NoOutputOrStageError,
-)
-from dvc.progress import Tqdm
+from dvc.exceptions import CheckoutError, CheckoutErrorSuggestGit, NoOutputOrStageError
 from dvc.utils import relpath
 
 from . import locked
@@ -19,16 +14,6 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-def _get_unused_links(repo):
-    used = [
-        out.fspath
-        for stage in repo.stages
-        for out in stage.outs
-        if out.scheme == "local"
-    ]
-    return repo.state.get_unused_links(used)
-
-
 def _fspath_dir(path):
     if not os.path.exists(str(path)):
         return str(path)
@@ -37,19 +22,22 @@ def _fspath_dir(path):
     return os.path.join(path, "") if os.path.isdir(path) else path
 
 
+def _remove_unused_links(repo):
+    used = [out.fspath for out in repo.index.outs if out.protocol == "local"]
+    unused = repo.state.get_unused_links(used, repo.fs)
+    ret = [_fspath_dir(u) for u in unused]
+    repo.state.remove_links(unused, repo.fs)
+    return ret
+
+
 def get_all_files_numbers(pairs):
-    return sum(
-        stage.get_all_files_number(filter_info) for stage, filter_info in pairs
-    )
+    return sum(stage.get_all_files_number(filter_info) for stage, filter_info in pairs)
 
 
 def _collect_pairs(
     self: "Repo", targets, with_deps: bool, recursive: bool
 ) -> Set["StageInfo"]:
-    from dvc.stage.exceptions import (
-        StageFileBadNameError,
-        StageFileDoesNotExistError,
-    )
+    from dvc.stage.exceptions import StageFileBadNameError, StageFileDoesNotExistError
 
     pairs: Set["StageInfo"] = set()
     for target in targets:
@@ -82,9 +70,9 @@ def checkout(
     allow_missing=False,
     **kwargs,
 ):
+    from dvc.fs.callbacks import Callback
 
-    unused = []
-    stats = {
+    stats: Dict[str, List[str]] = {
         "added": [],
         "deleted": [],
         "modified": [],
@@ -92,23 +80,23 @@ def checkout(
     }
     if not targets:
         targets = [None]
-        unused = _get_unused_links(self)
-
-    stats["deleted"] = [_fspath_dir(u) for u in unused]
-    self.state.remove_links(unused)
+        stats["deleted"] = _remove_unused_links(self)
 
     if isinstance(targets, str):
         targets = [targets]
 
     pairs = _collect_pairs(self, targets, with_deps, recursive)
     total = get_all_files_numbers(pairs)
-    with Tqdm(
-        total=total, unit="file", desc="Checkout", disable=total == 0
-    ) as pbar:
+    with Callback.as_tqdm_callback(
+        unit="file",
+        desc="Checkout",
+        disable=total == 0,
+    ) as cb:
+        cb.set_size(total)
         for stage, filter_info in pairs:
             result = stage.checkout(
                 force=force,
-                progress_callback=pbar.update_msg,
+                progress_callback=cb,
                 relink=relink,
                 filter_info=filter_info,
                 allow_missing=allow_missing,
