@@ -2,12 +2,8 @@ import logging
 from typing import Optional
 
 from dvc.repo import locked
-from dvc.repo.experiments.base import (
-    EXEC_NAMESPACE,
-    EXPS_NAMESPACE,
-    EXPS_STASH,
-    ExpRefInfo,
-)
+
+from .utils import exp_refs, remove_exp_refs
 
 logger = logging.getLogger(__name__)
 
@@ -19,6 +15,7 @@ def gc(
     all_tags: Optional[bool] = False,
     all_commits: Optional[bool] = False,
     workspace: Optional[bool] = False,
+    commit_date: Optional[str] = None,
     queued: Optional[bool] = False,
 ):
     keep_revs = set(
@@ -26,6 +23,7 @@ def gc(
             all_branches=all_branches,
             all_tags=all_tags,
             all_commits=all_commits,
+            commit_date=commit_date,
             sha_only=True,
         )
     )
@@ -35,21 +33,21 @@ def gc(
     if not keep_revs:
         return 0
 
-    removed = 0
-    for ref in repo.scm.iter_refs(EXPS_NAMESPACE):
-        if ref.startswith(EXEC_NAMESPACE) or ref == EXPS_STASH:
-            continue
-        ref_info = ExpRefInfo.from_ref(ref)
-        if ref_info.baseline_sha not in keep_revs:
-            repo.scm.remove_ref(ref)
-            removed += 1
+    to_remove = [
+        ref_info
+        for ref_info in exp_refs(repo.scm)
+        if ref_info.baseline_sha not in keep_revs
+    ]
+    remove_exp_refs(repo.scm, to_remove)
+    removed = len(to_remove)
 
     delete_stashes = []
-    for _, entry in repo.experiments.stash_revs.items():
+    stash = repo.experiments.celery_queue.stash
+    for stash_rev, entry in stash.stash_revs.items():
         if not queued or entry.baseline_rev not in keep_revs:
-            delete_stashes.append(entry.index)
-    for index in sorted(delete_stashes, reverse=True):
-        repo.experiments.stash.drop(index)
+            delete_stashes.append(stash_rev)
+    if delete_stashes:
+        repo.experiments.celery_queue.remove(delete_stashes)
     removed += len(delete_stashes)
 
     return removed
