@@ -1,11 +1,18 @@
 from funcy import first
 
 
+def test_diff_no_commits(tmp_dir):
+    from dvc.repo import Repo
+    from dvc.scm import Git
+
+    git = Git.init(tmp_dir.fs_path)
+    assert git.no_commits
+
+    assert Repo.init().experiments.diff() == {}
+
+
 def test_diff_empty(tmp_dir, scm, dvc, exp_stage):
-    assert dvc.experiments.diff() == {
-        "params": {},
-        "metrics": {},
-    }
+    assert dvc.experiments.diff() == {"params": {}, "metrics": {}}
 
 
 def test_diff_head(tmp_dir, scm, dvc, exp_stage):
