@@ -1,18 +1,14 @@
 import logging
 import os
 
-import mock
 import pytest
 
-from dvc.cache import Cache
-from dvc.exceptions import ConfirmRemoveError, PathMissingError
-from dvc.external_repo import IsADVCRepoError
-from dvc.main import main
+from dvc.cachemgr import CacheManager
+from dvc.cli import main
+from dvc.fs import system
 from dvc.repo import Repo
 from dvc.repo.get import GetDVCFileError
-from dvc.system import System
-from dvc.utils.fs import makedirs
-from tests.unit.tree.test_repo import make_subrepo
+from dvc.testing.tmp_dir import make_subrepo
 
 
 def test_get_repo_file(tmp_dir, erepo_dir):
@@ -28,52 +24,17 @@ def test_get_repo_file(tmp_dir, erepo_dir):
 def test_get_repo_file_replace_without_confirmation(tmp_dir, erepo_dir):
     with erepo_dir.chdir():
         erepo_dir.dvc_gen("file", "contents", commit="create file")
-        erepo_dir.dvc_gen(
-            "file2", "something different", commit="create file2"
-        )
+        erepo_dir.dvc_gen("file2", "something different", commit="create file2")
 
     Repo.get(os.fspath(erepo_dir), "file", "file_imported")
     # getting another file with a name that already exists in Repo.
-    with pytest.raises(ConfirmRemoveError):
+    with pytest.raises(FileExistsError):
         Repo.get(os.fspath(erepo_dir), "file2", "file_imported")
 
     assert os.path.isfile("file_imported")
     assert (tmp_dir / "file_imported").read_text() == "contents"
 
 
-def test_get_repo_file_replace_yes(tmp_dir, erepo_dir):
-    with erepo_dir.chdir():
-        erepo_dir.dvc_gen("file", "contents", commit="create file")
-        erepo_dir.dvc_gen(
-            "file2", "something different", commit="create file2"
-        )
-
-    Repo.get(os.fspath(erepo_dir), "file", "file_imported")
-    # getting another file with a name that already exists in Repo.
-    with mock.patch("dvc.prompt.confirm", return_value=True):
-        Repo.get(os.fspath(erepo_dir), "file2", "file_imported")
-
-    assert os.path.isfile("file_imported")
-    assert (tmp_dir / "file_imported").read_text() == "something different"
-
-
-def test_get_repo_file_replace_no(tmp_dir, erepo_dir):
-    with erepo_dir.chdir():
-        erepo_dir.dvc_gen("file", "contents", commit="create file")
-        erepo_dir.dvc_gen(
-            "file2", "something different", commit="create file2"
-        )
-
-    Repo.get(os.fspath(erepo_dir), "file", "file_imported")
-    # getting another file with a name that already exists in Repo.
-    with mock.patch("dvc.prompt.confirm", return_value=False):
-        with pytest.raises(ConfirmRemoveError):
-            Repo.get(os.fspath(erepo_dir), "file2", "file_imported")
-
-    assert os.path.isfile("file_imported")
-    assert (tmp_dir / "file_imported").read_text() == "contents"
-
-
 def test_get_repo_dir(tmp_dir, erepo_dir):
     with erepo_dir.chdir():
         erepo_dir.dvc_gen({"dir": {"file": "contents"}}, commit="create dir")
@@ -104,9 +65,7 @@ def test_get_git_dir(tmp_dir, erepo):
     src = "some_directory"
     dst = "some_directory_imported"
 
-    erepo.scm_gen(
-        {src: {"dir": {"file.txt": "hello"}}}, commit="add a regular dir"
-    )
+    erepo.scm_gen({src: {"dir": {"file.txt": "hello"}}}, commit="add a regular dir")
 
     Repo.get(os.fspath(erepo), src, dst)
 
@@ -117,16 +76,16 @@ def test_cache_type_is_properly_overridden(tmp_dir, erepo_dir):
     with erepo_dir.chdir():
         with erepo_dir.dvc.config.edit() as conf:
             conf["cache"]["type"] = "symlink"
-        erepo_dir.dvc.cache = Cache(erepo_dir.dvc)
+        erepo_dir.dvc.cache = CacheManager(erepo_dir.dvc)
         erepo_dir.scm_add(
             [erepo_dir.dvc.config.files["repo"]], "set cache type to symlinks"
         )
         erepo_dir.dvc_gen("file", "contents", "create file")
-    assert System.is_symlink(erepo_dir / "file")
+    assert system.is_symlink(erepo_dir / "file")
 
     Repo.get(os.fspath(erepo_dir), "file", "file_imported")
 
-    assert not System.is_symlink("file_imported")
+    assert not system.is_symlink("file_imported")
     assert (tmp_dir / "file_imported").read_text() == "contents"
 
 
@@ -160,9 +119,7 @@ def test_get_full_dvc_path(tmp_dir, erepo_dir, tmp_path_factory):
         erepo_dir.dvc.add(os.fspath(external_data), external=True)
         erepo_dir.scm_add("ext_data.dvc", commit="add external data")
 
-    Repo.get(
-        os.fspath(erepo_dir), os.fspath(external_data), "ext_data_imported"
-    )
+    Repo.get(os.fspath(erepo_dir), os.fspath(external_data), "ext_data_imported")
     assert (tmp_dir / "ext_data_imported").read_text() == "ext_data"
 
 
@@ -187,17 +144,17 @@ def test_non_cached_output(tmp_dir, erepo_dir):
 
 # https://github.com/iterative/dvc/pull/2837#discussion_r352123053
 def test_absolute_file_outside_repo(tmp_dir, erepo_dir):
-    with pytest.raises(PathMissingError):
+    with pytest.raises(FileNotFoundError):
         Repo.get(os.fspath(erepo_dir), "/root/")
 
 
 def test_absolute_file_outside_git_repo(tmp_dir, git_dir):
-    with pytest.raises(PathMissingError):
+    with pytest.raises(FileNotFoundError):
         Repo.get(os.fspath(git_dir), "/root/")
 
 
 def test_unknown_path(tmp_dir, erepo_dir):
-    with pytest.raises(PathMissingError):
+    with pytest.raises(FileNotFoundError):
         Repo.get(os.fspath(erepo_dir), "a_non_existing_file")
 
 
@@ -206,7 +163,7 @@ def test_get_to_dir(tmp_dir, erepo_dir, dname):
     with erepo_dir.chdir():
         erepo_dir.dvc_gen("file", "contents", commit="create file")
 
-    makedirs(dname, exist_ok=True)
+    os.makedirs(dname, exist_ok=True)
 
     Repo.get(os.fspath(erepo_dir), "file", dname)
 
@@ -247,9 +204,7 @@ def test_get_file_from_dir(tmp_dir, erepo_dir):
     assert (tmp_dir / "subdir" / "foo").read_text() == "foo"
     assert (tmp_dir / "subdir" / "bar").read_text() == "bar"
 
-    Repo.get(
-        os.fspath(erepo_dir), os.path.join("dir", "subdir", "foo"), out="X"
-    )
+    Repo.get(os.fspath(erepo_dir), os.path.join("dir", "subdir", "foo"), out="X")
     assert (tmp_dir / "X").read_text() == "foo"
 
 
@@ -289,20 +244,18 @@ def test_get_url_git_only_repo(tmp_dir, scm, caplog):
         assert "failed to show URL" in caplog.text
 
 
-def test_get_pipeline_tracked_outs(
-    tmp_dir, dvc, scm, git_dir, run_copy, local_remote
-):
-    from dvc.dvcfile import PIPELINE_FILE, PIPELINE_LOCK
+def test_get_pipeline_tracked_outs(tmp_dir, dvc, scm, git_dir, run_copy, local_remote):
+    from dvc.dvcfile import LOCK_FILE, PROJECT_FILE
 
     tmp_dir.gen("foo", "foo")
     run_copy("foo", "bar", name="copy-foo-bar")
     dvc.push()
 
-    dvc.scm.add([PIPELINE_FILE, PIPELINE_LOCK])
+    dvc.scm.add([PROJECT_FILE, LOCK_FILE])
     dvc.scm.commit("add pipeline stage")
 
     with git_dir.chdir():
-        Repo.get("file:///{}".format(os.fspath(tmp_dir)), "bar", out="baz")
+        Repo.get(f"file://{tmp_dir.as_posix()}", "bar", out="baz")
         assert (git_dir / "baz").read_text() == "foo"
 
 
@@ -345,17 +298,23 @@ def test_granular_get_from_subrepos(tmp_dir, erepo_dir):
     assert (tmp_dir / "out").read_text() == "bar"
 
 
-def test_try_to_get_complete_repo(tmp_dir, dvc, erepo_dir):
+def test_get_complete_repo(tmp_dir, dvc, erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen({"foo": "foo"}, commit="add foo")
+
     subrepo = erepo_dir / "subrepo"
     make_subrepo(subrepo, erepo_dir.scm)
     with subrepo.chdir():
         subrepo.dvc_gen({"dir": {"bar": "bar"}}, commit="files in subrepo")
 
-    expected_message = "Cannot fetch a complete DVC repository"
-    with pytest.raises(IsADVCRepoError) as exc_info:
-        Repo.get(os.fspath(erepo_dir), "subrepo", out="out")
-    assert f"{expected_message} 'subrepo'" == str(exc_info.value)
+    Repo.get(os.fspath(erepo_dir), "subrepo", out="out_sub")
+    assert (tmp_dir / "out_sub").read_text() == {
+        ".gitignore": "/dir\n",
+        "dir": {"bar": "bar"},
+    }
 
-    with pytest.raises(IsADVCRepoError) as exc_info:
-        Repo.get(os.fspath(erepo_dir), ".", out="out")
-    assert expected_message == str(exc_info.value)
+    Repo.get(os.fspath(erepo_dir), ".", out="out")
+    assert (tmp_dir / "out").read_text() == {
+        ".gitignore": "/foo\n",
+        "foo": "foo",
+    }
