@@ -3,18 +3,17 @@ import os
 
 import pytest
 from funcy import first
-from mock import patch
 
-import dvc.data_cloud as cloud
-from dvc.cache import Cache
+from dvc.annotations import Annotation
+from dvc.cachemgr import CacheManager
 from dvc.config import NoRemoteError
-from dvc.dvcfile import Dvcfile
+from dvc.dvcfile import load_file
 from dvc.exceptions import DownloadError, PathMissingError
-from dvc.external_repo import IsADVCRepoError
+from dvc.fs import system
+from dvc.scm import Git
 from dvc.stage.exceptions import StagePathNotFoundError
-from dvc.system import System
-from dvc.utils.fs import makedirs, remove
-from tests.unit.tree.test_repo import make_subrepo
+from dvc.testing.tmp_dir import make_subrepo
+from dvc.utils.fs import remove
 
 
 def test_import(tmp_dir, scm, dvc, erepo_dir):
@@ -25,7 +24,7 @@ def test_import(tmp_dir, scm, dvc, erepo_dir):
 
     assert os.path.isfile("foo_imported")
     assert (tmp_dir / "foo_imported").read_text() == "foo content"
-    assert scm.repo.git.check_ignore("foo_imported")
+    assert scm.is_ignored("foo_imported")
     assert stage.deps[0].def_repo == {
         "url": os.fspath(erepo_dir),
         "rev_lock": erepo_dir.scm.get_rev(),
@@ -42,14 +41,14 @@ def test_import_git_file(tmp_dir, scm, dvc, git_dir, src_is_dvc):
     stage = tmp_dir.dvc.imp(os.fspath(git_dir), "src", "dst")
 
     assert (tmp_dir / "dst").read_text() == "hello"
-    assert tmp_dir.scm.repo.git.check_ignore(os.fspath(tmp_dir / "dst"))
+    assert tmp_dir.scm.is_ignored(os.fspath(tmp_dir / "dst"))
     assert stage.deps[0].def_repo == {
         "url": os.fspath(git_dir),
         "rev_lock": git_dir.scm.get_rev(),
     }
 
 
-def test_import_cached_file(erepo_dir, tmp_dir, dvc, scm, monkeypatch):
+def test_import_cached_file(mocker, erepo_dir, tmp_dir, dvc, scm, monkeypatch):
     src = "some_file"
     dst = "some_file_imported"
 
@@ -60,8 +59,8 @@ def test_import_cached_file(erepo_dir, tmp_dir, dvc, scm, monkeypatch):
     (tmp_dir / dst).unlink()
 
     remote_exception = NoRemoteError("dvc import")
-    with patch.object(cloud.DataCloud, "pull", side_effect=remote_exception):
-        tmp_dir.dvc.imp(os.fspath(erepo_dir), src, dst)
+    mocker.patch.object(dvc.cloud, "get_remote_odb", side_effect=remote_exception)
+    tmp_dir.dvc.imp(os.fspath(erepo_dir), src, dst)
 
     assert (tmp_dir / dst).is_file()
     assert filecmp.cmp(erepo_dir / src, tmp_dir / dst, shallow=False)
@@ -77,7 +76,7 @@ def test_import_git_dir(tmp_dir, scm, dvc, git_dir, src_is_dvc):
     stage = dvc.imp(os.fspath(git_dir), "src", "dst")
 
     assert (tmp_dir / "dst").read_text() == {"file.txt": "hello"}
-    assert tmp_dir.scm.repo.git.check_ignore(os.fspath(tmp_dir / "dst"))
+    assert tmp_dir.scm.is_ignored(os.fspath(tmp_dir / "dst"))
     assert stage.deps[0].def_repo == {
         "url": os.fspath(git_dir),
         "rev_lock": git_dir.scm.get_rev(),
@@ -91,7 +90,7 @@ def test_import_dir(tmp_dir, scm, dvc, erepo_dir):
     stage = dvc.imp(os.fspath(erepo_dir), "dir", "dir_imported")
 
     assert (tmp_dir / "dir_imported").read_text() == {"foo": "foo content"}
-    assert scm.repo.git.check_ignore("dir_imported")
+    assert scm.is_ignored("dir_imported")
     assert stage.deps[0].def_repo == {
         "url": os.fspath(erepo_dir),
         "rev_lock": erepo_dir.scm.get_rev(),
@@ -114,7 +113,7 @@ def test_import_file_from_dir(tmp_dir, scm, dvc, erepo_dir):
     stage = dvc.imp(os.fspath(erepo_dir), os.path.join("dir", "1"))
 
     assert (tmp_dir / "1").read_text() == "1"
-    assert scm.repo.git.check_ignore("1")
+    assert scm.is_ignored("1")
     assert stage.deps[0].def_repo == {
         "url": os.fspath(erepo_dir),
         "rev_lock": erepo_dir.scm.get_rev(),
@@ -129,9 +128,7 @@ def test_import_file_from_dir(tmp_dir, scm, dvc, erepo_dir):
     assert (tmp_dir / "subdir" / "bar").read_text() == "bar"
     assert (tmp_dir / "subdir.dvc").exists()
 
-    dvc.imp(
-        os.fspath(erepo_dir), os.path.join("dir", "subdir", "foo"), out="X"
-    )
+    dvc.imp(os.fspath(erepo_dir), os.path.join("dir", "subdir", "foo"), out="X")
     assert (tmp_dir / "X").read_text() == "foo"
     assert (tmp_dir / "X.dvc").exists()
 
@@ -164,18 +161,16 @@ def test_import_non_cached(erepo_dir, tmp_dir, dvc, scm):
 
     with erepo_dir.chdir():
         erepo_dir.dvc.run(
-            cmd=f"echo hello > {src}", outs_no_cache=[src], single_stage=True,
+            cmd=f"echo hello > {src}", outs_no_cache=[src], single_stage=True
         )
 
-    erepo_dir.scm_add(
-        [os.fspath(erepo_dir / src)], commit="add a non-cached out"
-    )
+    erepo_dir.scm_add([os.fspath(erepo_dir / src)], commit="add a non-cached out")
 
     stage = tmp_dir.dvc.imp(os.fspath(erepo_dir), src, dst)
 
     assert (tmp_dir / dst).is_file()
     assert filecmp.cmp(erepo_dir / src, tmp_dir / dst, shallow=False)
-    assert tmp_dir.scm.repo.git.check_ignore(dst)
+    assert tmp_dir.scm.is_ignored(dst)
     assert stage.deps[0].def_repo == {
         "url": os.fspath(erepo_dir),
         "rev_lock": erepo_dir.scm.get_rev(),
@@ -191,7 +186,7 @@ def test_import_rev(tmp_dir, scm, dvc, erepo_dir):
     stage = dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported", rev="branch")
 
     assert (tmp_dir / "foo_imported").read_text() == "foo content"
-    assert scm.repo.git.check_ignore("foo_imported")
+    assert scm.is_ignored("foo_imported")
     assert stage.deps[0].def_repo == {
         "url": os.fspath(erepo_dir),
         "rev": "branch",
@@ -204,7 +199,7 @@ def test_pull_imported_stage(tmp_dir, dvc, erepo_dir):
         erepo_dir.dvc_gen("foo", "foo content", commit="create foo")
     dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported")
 
-    dst_stage = Dvcfile(dvc, "foo_imported.dvc").stage
+    dst_stage = load_file(dvc, "foo_imported.dvc").stage
     dst_cache = dst_stage.outs[0].cache_path
 
     remove("foo_imported")
@@ -215,23 +210,81 @@ def test_pull_imported_stage(tmp_dir, dvc, erepo_dir):
     assert os.path.isfile(dst_cache)
 
 
+def test_import_no_download(tmp_dir, scm, dvc, erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen("foo", "foo content", commit="create foo")
+
+    dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported", no_download=True)
+
+    assert not os.path.exists("foo_imported")
+
+    dst_stage = load_file(dvc, "foo_imported.dvc").stage
+
+    assert dst_stage.deps[0].def_repo == {
+        "url": os.fspath(erepo_dir),
+        "rev_lock": erepo_dir.scm.get_rev(),
+    }
+    assert scm.is_ignored("foo_imported")
+
+
+def test_pull_import_no_download(tmp_dir, scm, dvc, erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.scm_gen(os.path.join("foo", "bar"), b"bar", commit="add bar")
+        erepo_dir.dvc_gen(os.path.join("foo", "baz"), b"baz contents", commit="add baz")
+        size = (
+            len(b"bar")
+            + len(b"baz contents")
+            + len((erepo_dir / "foo" / ".gitignore").read_bytes())
+        )
+
+    dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported", no_download=True)
+
+    dvc.pull(["foo_imported.dvc"])
+    assert os.path.exists("foo_imported")
+
+    stage = load_file(dvc, "foo_imported.dvc").stage
+
+    assert stage.outs[0].hash_info.value == "bdb8641831d8fcb03939637e09011c21.dir"
+
+    assert stage.outs[0].meta.size == size
+    assert stage.outs[0].meta.nfiles == 3
+    assert stage.outs[0].meta.isdir
+
+
+def test_pull_import_no_download_rev_lock(
+    tmp_dir,
+    dvc,
+    erepo_dir,
+):
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen("foo", "foo content", commit="add")
+
+    dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported", no_download=True)
+
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen("foo", "modified foo content", commit="modify foo")
+
+    dvc.pull(["foo_imported.dvc"])
+    assert (tmp_dir / "foo_imported").read_text() == "foo content"
+
+
 def test_cache_type_is_properly_overridden(tmp_dir, scm, dvc, erepo_dir):
     with erepo_dir.chdir():
         with erepo_dir.dvc.config.edit() as conf:
             conf["cache"]["type"] = "symlink"
-        erepo_dir.dvc.cache = Cache(erepo_dir.dvc)
+        erepo_dir.dvc.cache = CacheManager(erepo_dir.dvc)
         erepo_dir.scm_add(
             [erepo_dir.dvc.config.files["repo"]],
             "set source repo cache type to symlink",
         )
         erepo_dir.dvc_gen("foo", "foo content", "create foo")
-    assert System.is_symlink(erepo_dir / "foo")
+    assert system.is_symlink(erepo_dir / "foo")
 
     dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported")
 
-    assert not System.is_symlink("foo_imported")
+    assert not system.is_symlink("foo_imported")
     assert (tmp_dir / "foo_imported").read_text() == "foo content"
-    assert scm.repo.git.check_ignore("foo_imported")
+    assert scm.is_ignored("foo_imported")
 
 
 def test_pull_imported_directory_stage(tmp_dir, dvc, erepo_dir):
@@ -241,33 +294,83 @@ def test_pull_imported_directory_stage(tmp_dir, dvc, erepo_dir):
     dvc.imp(os.fspath(erepo_dir), "dir", "dir_imported")
 
     remove("dir_imported")
-    remove(dvc.cache.local.cache_dir)
+    dvc.cache.local.clear()
 
     dvc.pull(["dir_imported.dvc"])
 
     assert (tmp_dir / "dir_imported").read_text() == {"foo": "foo content"}
 
 
-def test_download_error_pulling_imported_stage(tmp_dir, dvc, erepo_dir):
+def test_pull_wildcard_imported_directory_stage(tmp_dir, dvc, erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen({"dir123": {"foo": "foo content"}}, commit="create dir")
+
+    dvc.imp(os.fspath(erepo_dir), "dir123", "dir_imported123")
+
+    remove("dir_imported123")
+    dvc.cache.local.clear()
+
+    dvc.pull(["dir_imported*.dvc"], glob=True)
+
+    assert (tmp_dir / "dir_imported123").read_text() == {"foo": "foo content"}
+
+
+def test_push_wildcard_from_bare_git_repo(
+    tmp_dir, make_tmp_dir, erepo_dir, local_cloud
+):
+    Git.init(tmp_dir.fs_path, bare=True).close()
+
+    erepo_dir.add_remote(config=local_cloud.config)
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen(
+            {
+                "dir123": {"foo": "foo content"},
+                "dirextra": {"extrafoo": "extra foo content"},
+            },
+            commit="initial",
+        )
+    erepo_dir.dvc.push([os.path.join(os.fspath(erepo_dir), "dire*")], glob=True)
+
+    erepo_dir.scm.gitpython.repo.create_remote("origin", os.fspath(tmp_dir))
+    erepo_dir.scm.gitpython.repo.remote("origin").push("master")
+
+    dvc_repo = make_tmp_dir("dvc-repo", scm=True, dvc=True)
+    with dvc_repo.chdir():
+        dvc_repo.dvc.imp(os.fspath(tmp_dir), "dirextra")
+
+        with pytest.raises(PathMissingError):
+            dvc_repo.dvc.imp(os.fspath(tmp_dir), "dir123")
+
+
+def test_download_error_pulling_imported_stage(mocker, tmp_dir, dvc, erepo_dir):
     with erepo_dir.chdir():
         erepo_dir.dvc_gen("foo", "foo content", commit="create foo")
     dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported")
 
-    dst_stage = Dvcfile(dvc, "foo_imported.dvc").stage
+    dst_stage = load_file(dvc, "foo_imported.dvc").stage
     dst_cache = dst_stage.outs[0].cache_path
 
     remove("foo_imported")
     remove(dst_cache)
 
-    with patch(
-        "dvc.tree.local.LocalTree._download", side_effect=Exception
-    ), pytest.raises(DownloadError):
+    def unreliable_download(_from_fs, from_info, _to_fs, to_info, **kwargs):
+        on_error = kwargs["on_error"]
+        assert on_error
+        if isinstance(from_info, str):
+            from_info = [from_info]
+        if isinstance(to_info, str):
+            to_info = [to_info]
+        for from_i, to_i in zip(from_info, to_info):
+            on_error(from_i, to_i, Exception())
+
+    mocker.patch("dvc_objects.fs.generic.transfer", unreliable_download)
+    with pytest.raises(DownloadError):
         dvc.pull(["foo_imported.dvc"])
 
 
 @pytest.mark.parametrize("dname", [".", "dir", "dir/subdir"])
 def test_import_to_dir(dname, tmp_dir, dvc, erepo_dir):
-    makedirs(dname, exist_ok=True)
+    os.makedirs(dname, exist_ok=True)
 
     with erepo_dir.chdir():
         erepo_dir.dvc_gen("foo", "foo content", commit="create foo")
@@ -293,6 +396,7 @@ def test_pull_non_workspace(tmp_dir, scm, dvc, erepo_dir):
     scm.tag("ref-to-branch")
 
     # Overwrite via import
+    (tmp_dir / "foo_imported").unlink()
     dvc.imp(os.fspath(erepo_dir), "foo", "foo_imported", rev="master")
 
     remove(stage.outs[0].cache_path)
@@ -317,7 +421,7 @@ def test_pull_no_rev_lock(erepo_dir, tmp_dir, dvc):
     assert "rev" not in stage.deps[0].def_repo
     stage.deps[0].def_repo.pop("rev_lock")
 
-    Dvcfile(dvc, stage.path).dump(stage)
+    load_file(dvc, stage.path).dump(stage)
 
     remove(stage.outs[0].cache_path)
     (tmp_dir / "foo_imported").unlink()
@@ -328,20 +432,16 @@ def test_pull_no_rev_lock(erepo_dir, tmp_dir, dvc):
     assert (tmp_dir / "foo_imported").read_text() == "contents"
 
 
-def test_import_from_bare_git_repo(
-    tmp_dir, make_tmp_dir, erepo_dir, local_cloud
-):
-    import git
-
-    git.Repo.init(os.fspath(tmp_dir), bare=True)
+def test_import_from_bare_git_repo(tmp_dir, make_tmp_dir, erepo_dir, local_cloud):
+    Git.init(tmp_dir.fs_path, bare=True).close()
 
     erepo_dir.add_remote(config=local_cloud.config)
     with erepo_dir.chdir():
         erepo_dir.dvc_gen({"foo": "foo"}, commit="initial")
     erepo_dir.dvc.push()
 
-    erepo_dir.scm.repo.create_remote("origin", os.fspath(tmp_dir))
-    erepo_dir.scm.repo.remote("origin").push("master")
+    erepo_dir.scm.gitpython.repo.create_remote("origin", os.fspath(tmp_dir))
+    erepo_dir.scm.gitpython.repo.remote("origin").push("master")
 
     dvc_repo = make_tmp_dir("dvc-repo", scm=True, dvc=True)
     with dvc_repo.chdir():
@@ -351,19 +451,17 @@ def test_import_from_bare_git_repo(
 def test_import_pipeline_tracked_outs(
     tmp_dir, dvc, scm, erepo_dir, run_copy, local_remote
 ):
-    from dvc.dvcfile import PIPELINE_FILE, PIPELINE_LOCK
+    from dvc.dvcfile import LOCK_FILE, PROJECT_FILE
 
     tmp_dir.gen("foo", "foo")
     run_copy("foo", "bar", name="copy-foo-bar")
     dvc.push()
 
-    dvc.scm.add([PIPELINE_FILE, PIPELINE_LOCK])
+    dvc.scm.add([PROJECT_FILE, LOCK_FILE])
     dvc.scm.commit("add pipeline stage")
 
     with erepo_dir.chdir():
-        erepo_dir.dvc.imp(
-            "file:///{}".format(os.fspath(tmp_dir)), "bar", out="baz"
-        )
+        erepo_dir.dvc.imp(f"file://{tmp_dir.as_posix()}", "bar", out="baz")
         assert (erepo_dir / "baz").read_text() == "foo"
 
 
@@ -398,7 +496,7 @@ def test_import_subrepos(tmp_dir, erepo_dir, dvc, scm, is_dvc, files):
     key = next(iter(files))
     path = str((subrepo / key).relative_to(erepo_dir))
 
-    stage = dvc.imp(os.fspath(erepo_dir), path, out="out",)
+    stage = dvc.imp(os.fspath(erepo_dir), path, out="out")
 
     assert (tmp_dir / "out").read_text() == files[key]
     assert stage.deps[0].def_path == path
@@ -426,9 +524,7 @@ def test_granular_import_from_subrepos(tmp_dir, dvc, erepo_dir):
 
 @pytest.mark.parametrize("is_dvc", [True, False])
 @pytest.mark.parametrize("files", [{"foo": "foo"}, {"dir": {"bar": "bar"}}])
-def test_pull_imported_stage_from_subrepos(
-    tmp_dir, dvc, erepo_dir, is_dvc, files
-):
+def test_pull_imported_stage_from_subrepos(tmp_dir, dvc, erepo_dir, is_dvc, files):
     subrepo = erepo_dir / "subrepo"
     make_subrepo(subrepo, erepo_dir.scm)
     gen = subrepo.dvc_gen if is_dvc else subrepo.scm_gen
@@ -440,9 +536,8 @@ def test_pull_imported_stage_from_subrepos(
     dvc.imp(os.fspath(erepo_dir), path, out="out")
 
     # clean everything
-    remove(dvc.cache.local.cache_dir)
+    dvc.cache.local.clear()
     remove("out")
-    makedirs(dvc.cache.local.cache_dir)
 
     stats = dvc.pull(["out.dvc"])
 
@@ -451,20 +546,26 @@ def test_pull_imported_stage_from_subrepos(
     assert (tmp_dir / "out").read_text() == files[key]
 
 
-def test_try_import_complete_repo(tmp_dir, dvc, erepo_dir):
+def test_import_complete_repo(tmp_dir, dvc, erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen({"foo": "foo"}, commit="add foo")
+
     subrepo = erepo_dir / "subrepo"
     make_subrepo(subrepo, erepo_dir.scm)
     with subrepo.chdir():
         subrepo.dvc_gen({"dir": {"bar": "bar"}}, commit="files in subrepo")
 
-    expected_message = "Cannot fetch a complete DVC repository"
-    with pytest.raises(IsADVCRepoError) as exc_info:
-        dvc.imp(os.fspath(erepo_dir), "subrepo", out="out")
-    assert f"{expected_message} 'subrepo'" == str(exc_info.value)
+    dvc.imp(os.fspath(erepo_dir), "subrepo", out="out_sub")
+    assert (tmp_dir / "out_sub").read_text() == {
+        ".gitignore": "/dir\n",
+        "dir": {"bar": "bar"},
+    }
 
-    with pytest.raises(IsADVCRepoError) as exc_info:
-        dvc.imp(os.fspath(erepo_dir), os.curdir, out="out")
-    assert expected_message == str(exc_info.value)
+    dvc.imp(os.fspath(erepo_dir), os.curdir, out="out")
+    assert (tmp_dir / "out").read_text() == {
+        ".gitignore": "/foo\n",
+        "foo": "foo",
+    }
 
 
 def test_import_with_no_exec(tmp_dir, dvc, erepo_dir):
@@ -475,3 +576,125 @@ def test_import_with_no_exec(tmp_dir, dvc, erepo_dir):
 
     dst = tmp_dir / "foo_imported"
     assert not dst.exists()
+
+
+def test_import_with_jobs(mocker, dvc, erepo_dir):
+    import dvc_data.hashfile.transfer as otransfer
+
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen(
+            {
+                "dir1": {
+                    "file1": "file1",
+                    "file2": "file2",
+                    "file3": "file3",
+                    "file4": "file4",
+                }
+            },
+            commit="init",
+        )
+
+    spy = mocker.spy(otransfer, "transfer")
+    dvc.imp(os.fspath(erepo_dir), "dir1", jobs=3)
+    # the first call will be retrieving dir cache for "dir1" w/jobs None
+    for _args, kwargs in spy.call_args_list[1:]:
+        assert kwargs.get("jobs") == 3
+
+
+def test_chained_import(tmp_dir, dvc, make_tmp_dir, erepo_dir, local_cloud):
+    erepo_dir.add_remote(config=local_cloud.config)
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen({"dir": {"foo": "foo", "bar": "bar"}}, commit="init")
+    erepo_dir.dvc.push()
+    remove(erepo_dir.dvc.cache.local.path)
+    remove(os.fspath(erepo_dir / "dir"))
+
+    erepo2 = make_tmp_dir("erepo2", scm=True, dvc=True)
+    with erepo2.chdir():
+        erepo2.dvc.imp(os.fspath(erepo_dir), "dir")
+        erepo2.scm.add("dir.dvc")
+        erepo2.scm.commit("import")
+    remove(erepo2.dvc.cache.local.path)
+    remove(os.fspath(erepo2 / "dir"))
+
+    dvc.imp(os.fspath(erepo2), "dir", "dir_imported")
+    dst = tmp_dir / "dir_imported"
+    assert (dst / "foo").read_text() == "foo"
+    assert (dst / "bar").read_text() == "bar"
+
+    dvc.cache.local.clear()
+    remove("dir_imported")
+
+    # pulled objects should come from the original upstream repo's remote,
+    # no cache or remote should be needed from the intermediate repo
+    dvc.pull(["dir_imported.dvc"])
+    assert not os.path.exists(erepo_dir.dvc.cache.local.path)
+    assert not os.path.exists(erepo2.dvc.cache.local.path)
+    assert (dst / "foo").read_text() == "foo"
+    assert (dst / "bar").read_text() == "bar"
+
+
+def test_circular_import(tmp_dir, dvc, scm, erepo_dir):
+    from dvc.exceptions import CircularImportError
+
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen({"dir": {"foo": "foo", "bar": "bar"}}, commit="init")
+
+    dvc.imp(os.fspath(erepo_dir), "dir", "dir_imported")
+    scm.add("dir_imported.dvc")
+    scm.commit("import")
+
+    with erepo_dir.chdir():
+        with pytest.raises(CircularImportError):
+            erepo_dir.dvc.imp(os.fspath(tmp_dir), "dir_imported", "circular_import")
+
+
+@pytest.mark.parametrize("paths", ([], ["dir"]))
+def test_parameterized_repo(tmp_dir, dvc, scm, erepo_dir, paths):
+    path = erepo_dir.joinpath(*paths)
+    path.mkdir(parents=True, exist_ok=True)
+    (path / "params.yaml").dump({"out": "foo"})
+    (path / "dvc.yaml").dump(
+        {
+            "stages": {
+                "train": {"cmd": "echo ${out} > ${out}", "outs": ["${out}"]},
+            }
+        }
+    )
+    path.gen({"foo": "foo"})
+    with path.chdir():
+        erepo_dir.dvc.commit(None, force=True)
+        erepo_dir.scm.add_commit(
+            ["params.yaml", "dvc.yaml", "dvc.lock", ".gitignore"],
+            message="init",
+        )
+
+    to_import = os.path.join(*paths, "foo")
+    stage = dvc.imp(os.fspath(erepo_dir), to_import, "foo_imported")
+
+    assert (tmp_dir / "foo_imported").read_text() == "foo"
+    assert stage.deps[0].def_repo == {
+        "url": os.fspath(erepo_dir),
+        "rev_lock": erepo_dir.scm.get_rev(),
+    }
+
+
+def test_import_with_annotations(M, tmp_dir, scm, dvc, erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.dvc_gen("foo", "foo content", commit="create foo")
+
+    annot = {
+        "desc": "foo desc",
+        "labels": ["l1", "l2"],
+        "type": "t1",
+        "meta": {"key": "value"},
+    }
+    stage = dvc.imp(os.fspath(erepo_dir), "foo", "foo", no_exec=True, **annot)
+    assert stage.outs[0].annot == Annotation(**annot)
+    assert (tmp_dir / "foo.dvc").parse() == M.dict(outs=[M.dict(**annot)])
+
+    # try to selectively update/overwrite some annotations
+    annot = {**annot, "type": "t2"}
+    stage = dvc.imp(os.fspath(erepo_dir), "foo", "foo", no_exec=True, type="t2")
+    assert stage.outs[0].annot == Annotation(**annot)
+    assert (tmp_dir / "foo.dvc").parse() == M.dict(outs=[M.dict(**annot)])
