@@ -1,53 +1,47 @@
 import logging
 import os
 
+import pytest
+
+from dvc.cli import main
 from dvc.config import Config
 from dvc.exceptions import InitError
-from dvc.main import main
 from dvc.repo import Repo as DvcRepo
-from tests.basic_env import TestDir, TestGit
 
 
-class TestInit(TestGit):
-    def _test_init(self):
-        self.assertTrue(os.path.exists(DvcRepo.DVC_DIR))
-        self.assertTrue(os.path.isdir(DvcRepo.DVC_DIR))
+def test_api_init(scm):
+    DvcRepo.init().close()
+    assert os.path.isdir(DvcRepo.DVC_DIR)
 
-    def test_api(self):
+
+def test_cli_init(scm):
+    ret = main(["init"])
+    assert ret == 0
+    assert os.path.isdir(DvcRepo.DVC_DIR)
+
+
+def test_double_init(scm):
+    ret = main(["init"])
+    assert ret == 0
+    assert os.path.isdir(DvcRepo.DVC_DIR)
+
+    ret = main(["init"])
+    assert ret != 0
+    assert os.path.isdir(DvcRepo.DVC_DIR)
+
+    ret = main(["init", "--force"])
+    assert ret == 0
+    assert os.path.isdir(DvcRepo.DVC_DIR)
+
+
+def test_init_no_scm_fail_api(tmp_dir):
+    with pytest.raises(InitError):
         DvcRepo.init()
 
-        self._test_init()
 
-    def test_cli(self):
-        ret = main(["init"])
-        self.assertEqual(ret, 0)
-
-        self._test_init()
-
-
-class TestDoubleInit(TestInit):
-    def test(self):
-        ret = main(["init"])
-        self.assertEqual(ret, 0)
-        self._test_init()
-
-        ret = main(["init"])
-        self.assertNotEqual(ret, 0)
-        self._test_init()
-
-        ret = main(["init", "--force"])
-        self.assertEqual(ret, 0)
-        self._test_init()
-
-
-class TestInitNoSCMFail(TestDir):
-    def test_api(self):
-        with self.assertRaises(InitError):
-            DvcRepo.init()
-
-    def test_cli(self):
-        ret = main(["init"])
-        self.assertNotEqual(ret, 0)
+def test_init_no_scm_fail_cli(tmp_dir):
+    ret = main(["init"])
+    assert ret != 0
 
 
 def test_init_no_scm_api(tmp_dir):
@@ -70,8 +64,8 @@ def test_init_quiet_should_not_display_welcome_screen(tmp_dir, scm, caplog):
     with caplog.at_level(logging.INFO, logger="dvc"):
         ret = main(["init", "--quiet"])
 
-        assert 0 == ret
-        assert "" == caplog.text
+        assert ret == 0
+        assert caplog.text == ""
 
 
 def test_allow_init_dvc_subdir(tmp_dir, scm, monkeypatch):
@@ -99,8 +93,8 @@ def test_subdir_init_no_option(tmp_dir, scm, monkeypatch, caplog):
         "{} is not tracked by any supported SCM tool (e.g. Git). "
         "Use `--no-scm` if you don't want to use any SCM or "
         "`--subdir` if initializing inside a subdirectory of a parent SCM "
-        "repository.".format(os.fspath(tmp_dir / "subdir"))
-    ) in caplog.text
+        "repository.".format(os.fspath(tmp_dir / "subdir")) in caplog.text
+    )
 
 
 def test_gen_dvcignore(tmp_dir):
@@ -124,4 +118,5 @@ def test_init_when_ignored_by_git(tmp_dir, scm, caplog):
         "for example, by adding '!.dvc' to .gitignore.".format(
             dvc_dir=tmp_dir / DvcRepo.DVC_DIR
         )
-    ) in caplog.text
+        in caplog.text
+    )
