@@ -7,7 +7,7 @@ import pytest
 
 from dvc.exceptions import PathMissingError
 from dvc.repo import Repo
-from dvc.scm.base import CloneError
+from dvc.scm import CloneError
 
 FS_STRUCTURE = {
     "README.md": "content",
@@ -42,13 +42,11 @@ def create_dvc_pipeline(tmp_dir, dvc):
     tmp_dir.scm_gen({"script.py": script}, commit="init")
     tmp_dir.dvc_gen({"dep": "content"}, commit="init dvc")
     dvc.run(
-        **{
-            "cmd": "python script.py {}".format(os.path.join("out", "file")),
-            "outs": [os.path.join("out", "file")],
-            "deps": ["dep"],
-            "fname": "out.dvc",
-            "single_stage": True,
-        }
+        cmd="python script.py {}".format(os.path.join("out", "file")),
+        outs=[os.path.join("out", "file")],
+        deps=["dep"],
+        fname="out.dvc",
+        single_stage=True,
     )
     tmp_dir.scm_add(["out.dvc"], commit="run")
     shutil.rmtree("out")
@@ -73,36 +71,6 @@ def test_ls_repo(tmp_dir, dvc, scm):
     )
 
 
-def test_ls_repo_with_color(tmp_dir, dvc, scm, mocker, monkeypatch, caplog):
-    import logging
-
-    from dvc.cli import parse_args
-
-    tmp_dir.scm_gen(FS_STRUCTURE, commit="init")
-    tmp_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
-
-    monkeypatch.setenv("LS_COLORS", "rs=0:di=01;34:*.xml=01;31:*.dvc=01;33:")
-    cli_args = parse_args(["list", os.fspath(tmp_dir)])
-    cmd = cli_args.func(cli_args)
-
-    caplog.clear()
-    mocker.patch("sys.stdout.isatty", return_value=True)
-    with caplog.at_level(logging.INFO, logger="dvc.command.ls"):
-        assert cmd.run() == 0
-
-    assert caplog.records[-1].msg == "\n".join(
-        [
-            ".dvcignore",
-            ".gitignore",
-            "README.md",
-            "\x1b[01;34mdata\x1b[0m",
-            "\x1b[01;34mmodel\x1b[0m",
-            "\x1b[01;31mstructure.xml\x1b[0m",
-            "\x1b[01;33mstructure.xml.dvc\x1b[0m",
-        ]
-    )
-
-
 def test_ls_repo_recursive(tmp_dir, dvc, scm):
     tmp_dir.scm_gen(FS_STRUCTURE, commit="init")
     tmp_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
@@ -147,6 +115,18 @@ def test_ls_repo_dvc_only_recursive(tmp_dir, dvc, scm):
     )
 
 
+def test_ls_repo_with_new_path_dir(tmp_dir, dvc, scm):
+    tmp_dir.scm_gen(FS_STRUCTURE, commit="init")
+    tmp_dir.dvc_gen({"mysub": {}}, commit="dvc")
+    tmp_dir.gen({"mysub/sub": {"foo": "content"}})
+
+    files = Repo.ls(os.fspath(tmp_dir), path="mysub/sub")
+    match_files(
+        files,
+        ((("foo",), False),),
+    )
+
+
 def test_ls_repo_with_path_dir(tmp_dir, dvc, scm):
     tmp_dir.scm_gen(FS_STRUCTURE, commit="init")
     tmp_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
@@ -168,9 +148,17 @@ def test_ls_repo_with_path_dir_dvc_only_empty(tmp_dir, dvc, scm):
     tmp_dir.scm_gen(FS_STRUCTURE, commit="init")
     tmp_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
     tmp_dir.scm_gen({"folder/.keep": "content"}, commit="add .keep")
+    tmp_dir.scm_gen({"empty_scm_folder/": {}}, commit="add scm empty")
+    tmp_dir.dvc_gen({"empty_dvc_folder": {}}, commit="empty dvc folder")
 
     with pytest.raises(PathMissingError):
-        Repo.ls(os.fspath(tmp_dir), path="folder", dvc_only=True)
+        Repo.ls(os.fspath(tmp_dir), path="not_exist_folder")
+
+    assert Repo.ls(os.fspath(tmp_dir), path="empty_scm_folder") == []
+
+    assert Repo.ls(os.fspath(tmp_dir), path="folder", dvc_only=True) == []
+
+    assert Repo.ls(os.fspath(tmp_dir), path="empty_dvc_folder", dvc_only=True) == []
 
 
 def test_ls_repo_with_path_subdir(tmp_dir, dvc, scm):
@@ -196,7 +184,7 @@ def test_ls_repo_with_path_subdir_dvc_only(tmp_dir, dvc, scm):
 
     path = os.path.join("data", "subcontent")
     files = Repo.ls(os.fspath(tmp_dir), path, dvc_only=True)
-    match_files(files, ((("data.xml",), True), (("statistics",), False),))
+    match_files(files, ((("data.xml",), True), (("statistics",), False)))
 
 
 def test_ls_repo_with_path_subdir_dvc_only_recursive(tmp_dir, dvc, scm):
@@ -205,9 +193,7 @@ def test_ls_repo_with_path_subdir_dvc_only_recursive(tmp_dir, dvc, scm):
 
     path = os.path.join("data", "subcontent")
     files = Repo.ls(os.fspath(tmp_dir), path, dvc_only=True, recursive=True)
-    match_files(
-        files, ((("data.xml",), True), (("statistics", "data.csv"), True),)
-    )
+    match_files(files, ((("data.xml",), True), (("statistics", "data.csv"), True)))
 
 
 def test_ls_repo_with_path_file_out(tmp_dir, dvc, scm):
@@ -303,12 +289,30 @@ def test_ls_repo_with_removed_dvc_dir_with_path_file(tmp_dir, dvc, scm):
     match_files(files, ((("file",), True),))
 
 
+def test_ls_repo_with_rev(erepo_dir):
+    with erepo_dir.chdir():
+        erepo_dir.scm_gen(FS_STRUCTURE, commit="init")
+        erepo_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
+
+    rev = erepo_dir.scm.list_all_commits()[1]
+    files = Repo.ls(os.fspath(erepo_dir), rev=rev)
+    match_files(
+        files,
+        (
+            ((".dvcignore",), False),
+            ((".gitignore",), False),
+            (("README.md",), False),
+            (("model",), False),
+        ),
+    )
+
+
 def test_ls_remote_repo(erepo_dir):
     with erepo_dir.chdir():
         erepo_dir.scm_gen(FS_STRUCTURE, commit="init")
         erepo_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
 
-    url = f"file://{erepo_dir}"
+    url = f"file://{erepo_dir.as_posix()}"
     files = Repo.ls(url)
     match_files(
         files,
@@ -329,7 +333,7 @@ def test_ls_remote_repo_recursive(erepo_dir):
         erepo_dir.scm_gen(FS_STRUCTURE, commit="init")
         erepo_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
 
-    url = f"file://{erepo_dir}"
+    url = f"file://{erepo_dir.as_posix()}"
     files = Repo.ls(url, recursive=True)
     match_files(
         files,
@@ -358,7 +362,7 @@ def test_ls_remote_git_only_repo_recursive(git_dir):
     with git_dir.chdir():
         git_dir.scm_gen(FS_STRUCTURE, commit="init")
 
-    url = f"file://{git_dir}"
+    url = f"file://{git_dir.as_posix()}"
     files = Repo.ls(url, recursive=True)
     match_files(
         files,
@@ -376,7 +380,7 @@ def test_ls_remote_repo_with_path_dir(erepo_dir):
         erepo_dir.scm_gen(FS_STRUCTURE, commit="init")
         erepo_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
 
-    url = f"file://{erepo_dir}"
+    url = f"file://{erepo_dir.as_posix()}"
     path = "model"
     files = Repo.ls(url, path)
     match_files(
@@ -397,7 +401,7 @@ def test_ls_remote_repo_with_rev(erepo_dir):
         erepo_dir.dvc_gen(DVC_STRUCTURE, commit="dvc")
 
     rev = erepo_dir.scm.list_all_commits()[1]
-    url = f"file://{erepo_dir}"
+    url = f"file://{erepo_dir.as_posix()}"
     files = Repo.ls(url, rev=rev)
     match_files(
         files,
@@ -416,7 +420,7 @@ def test_ls_remote_repo_with_rev_recursive(erepo_dir):
         erepo_dir.scm_gen(FS_STRUCTURE, commit="init")
 
     rev = erepo_dir.scm.list_all_commits()[1]
-    url = f"file://{erepo_dir}"
+    url = f"file://{erepo_dir.as_posix()}"
     files = Repo.ls(url, rev=rev, recursive=True)
     match_files(
         files,
@@ -447,11 +451,11 @@ def test_ls_not_existed_url():
 
 
 def test_ls_shows_pipeline_tracked_outs(tmp_dir, dvc, scm, run_copy):
-    from dvc.dvcfile import PIPELINE_FILE, PIPELINE_LOCK
+    from dvc.dvcfile import LOCK_FILE, PROJECT_FILE
 
     tmp_dir.gen("foo", "foo")
     run_copy("foo", "bar", name="copy-foo-bar")
-    dvc.scm.add([PIPELINE_FILE, PIPELINE_LOCK])
+    dvc.scm.add([PROJECT_FILE, LOCK_FILE])
     dvc.scm.commit("add pipeline stage")
 
     files = Repo.ls(os.curdir, dvc_only=True)
@@ -473,15 +477,15 @@ def test_ls_granular(erepo_dir):
 
     entries = Repo.ls(os.fspath(erepo_dir), os.path.join("dir", "subdir"))
     assert entries == [
-        {"isout": False, "isdir": False, "isexec": False, "path": "bar"},
-        {"isout": False, "isdir": False, "isexec": False, "path": "foo"},
+        {"isout": True, "isdir": False, "isexec": False, "path": "bar"},
+        {"isout": True, "isdir": False, "isexec": False, "path": "foo"},
     ]
 
     entries = Repo.ls(os.fspath(erepo_dir), "dir")
     assert entries == [
-        {"isout": False, "isdir": False, "isexec": False, "path": "1"},
-        {"isout": False, "isdir": False, "isexec": False, "path": "2"},
-        {"isout": False, "isdir": True, "isexec": False, "path": "subdir"},
+        {"isout": True, "isdir": False, "isexec": False, "path": "1"},
+        {"isout": True, "isdir": False, "isexec": False, "path": "2"},
+        {"isout": True, "isdir": True, "isexec": False, "path": "subdir"},
     ]
 
 
@@ -500,18 +504,20 @@ def test_ls_target(erepo_dir, use_scm):
             commit="create dir",
         )
 
+    isout = not use_scm
+
     def _ls(path):
         return Repo.ls(os.fspath(erepo_dir), path)
 
     assert _ls(os.path.join("dir", "1")) == [
-        {"isout": False, "isdir": False, "isexec": False, "path": "1"}
+        {"isout": isout, "isdir": False, "isexec": False, "path": "1"}
     ]
     assert _ls(os.path.join("dir", "subdir", "foo")) == [
-        {"isout": False, "isdir": False, "isexec": False, "path": "foo"}
+        {"isout": isout, "isdir": False, "isexec": False, "path": "foo"}
     ]
     assert _ls(os.path.join("dir", "subdir")) == [
-        {"isdir": False, "isexec": 0, "isout": False, "path": "bar"},
-        {"isdir": False, "isexec": 0, "isout": False, "path": "foo"},
+        {"isdir": False, "isexec": 0, "isout": isout, "path": "bar"},
+        {"isdir": False, "isexec": 0, "isout": isout, "path": "foo"},
     ]
 
 
@@ -536,23 +542,46 @@ def test_subrepo(dvc_top_level, erepo):
             if hasattr(repo, "dvc"):
                 repo.dvc_gen(dvc_files, commit=f"dvc track for {repo}")
 
-    def _list_files(path=None):
-        return set(map(itemgetter("path"), Repo.ls(os.fspath(erepo), path)))
+    def _list_files(repo, path=None):
+        return set(map(itemgetter("path"), Repo.ls(os.fspath(repo), path)))
 
     extras = {".dvcignore", ".gitignore"}
     git_tracked_outputs = {"bar.txt", "scm_dir"}
     dvc_files = {"dvc_dir", "foo.txt", "foo.txt.dvc", "dvc_dir.dvc"}
     common_outputs = git_tracked_outputs | extras | dvc_files
 
-    top_level_outputs = (
-        common_outputs if dvc_top_level else git_tracked_outputs
-    )
-    assert _list_files() == top_level_outputs | {"subrepo"}
-    assert _list_files("subrepo") == common_outputs
-
-    assert _list_files("scm_dir") == {"ipsum"}
-    assert _list_files("subrepo/scm_dir") == {"ipsum"}
-
+    top_level_outputs = common_outputs if dvc_top_level else git_tracked_outputs
+    assert _list_files(erepo) == top_level_outputs
+    assert _list_files(erepo, "scm_dir") == {"ipsum"}
     if dvc_top_level:
-        assert _list_files("dvc_dir") == {"lorem"}
-    assert _list_files("subrepo/dvc_dir") == {"lorem"}
+        assert _list_files(erepo, "dvc_dir") == {"lorem"}
+
+    assert _list_files(subrepo, ".") == common_outputs
+    assert _list_files(subrepo, "scm_dir") == {"ipsum"}
+    assert _list_files(subrepo, "dvc_dir") == {"lorem"}
+
+
+def test_broken_symlink(tmp_dir, dvc):
+    from dvc.fs import system
+
+    tmp_dir.gen("file", "content")
+    system.symlink("file", "link")
+
+    os.remove("file")
+
+    entries = Repo.ls(os.fspath(tmp_dir))
+
+    assert entries == [
+        {
+            "isout": False,
+            "isdir": False,
+            "isexec": False,
+            "path": ".dvcignore",
+        },
+        {
+            "isout": False,
+            "isdir": False,
+            "isexec": False,
+            "path": "link",
+        },
+    ]
