@@ -1,79 +1,66 @@
 import errno
+import itertools
 import os
 import stat
+from unittest.mock import patch
 
 import configobj
 import pytest
-from mock import patch
 
+from dvc.cli import main
 from dvc.config import Config
-from dvc.dir_info import DirInfo
 from dvc.exceptions import DownloadError, RemoteCacheRequiredError, UploadError
-from dvc.main import main
-from dvc.path_info import PathInfo
-from dvc.tree.base import BaseTree
-from dvc.tree.local import LocalTree
 from dvc.utils.fs import remove
-from tests.basic_env import TestDvc
-from tests.remotes import Local
 
 
-class TestRemote(TestDvc):
-    def test(self):
-        remotes = ["a", "b", "c"]
+def test_remote(dvc):
+    remotes = ["a", "b", "c"]
 
-        self.assertEqual(main(["remote", "list"]), 0)
-        self.assertNotEqual(main(["remote", "remove", remotes[0]]), 0)
+    assert main(["remote", "list"]) == 0
+    assert main(["remote", "remove", remotes[0]]) != 0
 
-        for r in remotes:
-            self.assertEqual(
-                main(["remote", "add", "--default", r, "s3://bucket/name"]), 0
-            )
+    for r in remotes:
+        assert main(["remote", "add", "--default", r, "s3://bucket/name"]) == 0
 
-        self.assertEqual(main(["remote", "list"]), 0)
+    assert main(["remote", "list"]) == 0
 
-        self.assertEqual(
-            main(["remote", "modify", remotes[0], "checksum_jobs", "1"]), 0
-        )
-        self.assertEqual(main(["remote", "remove", remotes[0]]), 0)
+    assert main(["remote", "modify", remotes[0], "checksum_jobs", "1"]) == 0
+    assert main(["remote", "remove", remotes[0]]) == 0
 
-        self.assertEqual(main(["remote", "list"]), 0)
+    assert main(["remote", "list"]) == 0
 
-    def test_relative_path(self):
-        dname = os.path.join("..", "path", "to", "dir")
-        ret = main(["remote", "add", "mylocal", dname])
-        self.assertEqual(ret, 0)
 
-        # NOTE: we are in the repo's root and config is in .dvc/, so
-        # dir path written to config should be just one level above.
-        rel = os.path.join("..", dname)
-        config = configobj.ConfigObj(self.dvc.config.files["repo"])
-        self.assertEqual(
-            config['remote "mylocal"']["url"], rel.replace("\\", "/")
-        )
+def test_remote_add_relative_path(dvc):
+    dname = os.path.join("..", "path", "to", "dir")
+    ret = main(["remote", "add", "mylocal", dname])
+    assert ret == 0
 
-    def test_overwrite(self):
-        remote_name = "a"
-        remote_url = "s3://bucket/name"
-        self.assertEqual(main(["remote", "add", remote_name, remote_url]), 0)
-        self.assertEqual(main(["remote", "add", remote_name, remote_url]), 251)
-        self.assertEqual(
-            main(["remote", "add", "-f", remote_name, remote_url]), 0
-        )
+    # NOTE: we are in the repo's root and config is in .dvc/, so
+    # dir path written to config should be just one level above.
+    rel = os.path.join("..", dname)
+    config = configobj.ConfigObj(dvc.config.files["repo"])
+    assert config['remote "mylocal"']["url"] == rel.replace("\\", "/")
 
-    def test_referencing_other_remotes(self):
-        assert main(["remote", "add", "foo", "ssh://localhost/"]) == 0
-        assert main(["remote", "add", "bar", "remote://foo/dvc-storage"]) == 0
 
-        config = configobj.ConfigObj(self.dvc.config.files["repo"])
-        assert config['remote "bar"']["url"] == "remote://foo/dvc-storage"
+def test_remote_overwrite(dvc):
+    remote_name = "a"
+    remote_url = "s3://bucket/name"
+    assert main(["remote", "add", remote_name, remote_url]) == 0
+    assert main(["remote", "add", remote_name, remote_url]) == 251
+    assert main(["remote", "add", "-f", remote_name, remote_url]) == 0
+
+
+def test_referencing_other_remotes(dvc):
+    assert main(["remote", "add", "foo", "ssh://localhost/"]) == 0
+    assert main(["remote", "add", "bar", "remote://foo/dvc-storage"]) == 0
+
+    config = configobj.ConfigObj(dvc.config.files["repo"])
+    assert config['remote "bar"']["url"] == "remote://foo/dvc-storage"
 
 
 def test_remove_default(tmp_dir, dvc):
     remote = "mys3"
-    assert (
-        main(["remote", "add", "--default", remote, "s3://bucket/name"]) == 0
-    )
+    assert main(["remote", "add", "--default", remote, "s3://bucket/name"]) == 0
     assert main(["remote", "modify", remote, "profile", "default"]) == 0
     assert main(["config", "--local", "core.remote", remote]) == 0
 
@@ -90,37 +77,35 @@ def test_remove_default(tmp_dir, dvc):
     assert local_config.get("core", {}).get("remote") is None
 
 
-class TestRemoteRemove(TestDvc):
-    def test(self):
-        ret = main(["config", "core.checksum_jobs", "1"])
-        self.assertEqual(ret, 0)
+def test_remote_remove(dvc):
+    ret = main(["config", "core.checksum_jobs", "1"])
+    assert ret == 0
 
-        remote = "mys3"
-        ret = main(["remote", "add", remote, "s3://bucket/name"])
-        self.assertEqual(ret, 0)
+    remote = "mys3"
+    ret = main(["remote", "add", remote, "s3://bucket/name"])
+    assert ret == 0
 
-        ret = main(["remote", "remove", remote])
-        self.assertEqual(ret, 0)
+    ret = main(["remote", "remove", remote])
+    assert ret == 0
 
 
-class TestRemoteDefault(TestDvc):
-    def test(self):
-        remote = "mys3"
-        ret = main(["remote", "add", "mys3", "s3://bucket/path"])
-        self.assertEqual(ret, 0)
+def test_remote_default_cmd(dvc):
+    remote = "mys3"
+    ret = main(["remote", "add", "mys3", "s3://bucket/path"])
+    assert ret == 0
 
-        ret = main(["remote", "default", "mys3"])
-        self.assertEqual(ret, 0)
-        config_file = os.path.join(self.dvc.dvc_dir, Config.CONFIG)
-        config = configobj.ConfigObj(config_file)
-        default = config["core"]["remote"]
-        self.assertEqual(default, remote)
+    ret = main(["remote", "default", "mys3"])
+    assert ret == 0
+    config_file = os.path.join(dvc.dvc_dir, Config.CONFIG)
+    config = configobj.ConfigObj(config_file)
+    default = config["core"]["remote"]
+    assert default == remote
 
-        ret = main(["remote", "default", "--unset"])
-        self.assertEqual(ret, 0)
-        config = configobj.ConfigObj(config_file)
-        default = config.get("core", {}).get("remote")
-        self.assertEqual(default, None)
+    ret = main(["remote", "default", "--unset"])
+    assert ret == 0
+    config = configobj.ConfigObj(config_file)
+    default = config.get("core", {}).get("remote")
+    assert default is None
 
 
 def test_show_default(dvc, capsys):
@@ -131,84 +116,111 @@ def test_show_default(dvc, capsys):
     assert out == "foo\n"
 
 
-class TestRemoteShouldHandleUppercaseRemoteName(TestDvc):
-    upper_case_remote_name = "UPPERCASEREMOTE"
+def test_upper_case_remote(tmp_dir, dvc, local_cloud):
+    remote_name = "UPPERCASEREMOTE"
 
-    def test(self):
-        remote_url = Local.get_storagepath()
-        ret = main(["remote", "add", self.upper_case_remote_name, remote_url])
-        self.assertEqual(ret, 0)
+    tmp_dir.gen("foo", "foo")
 
-        ret = main(["add", self.FOO])
-        self.assertEqual(ret, 0)
+    ret = main(["remote", "add", remote_name, local_cloud.url])
+    assert ret == 0
 
-        ret = main(["push", "-r", self.upper_case_remote_name])
-        self.assertEqual(ret, 0)
+    ret = main(["push", "-r", remote_name])
+    assert ret == 0
 
 
 def test_dir_hash_should_be_key_order_agnostic(tmp_dir, dvc):
+    from dvc_data.hashfile.build import build
+    from dvc_data.hashfile.tree import Tree
+
     tmp_dir.gen({"data": {"1": "1 content", "2": "2 content"}})
 
-    path_info = PathInfo("data")
+    path = (tmp_dir / "data").fs_path
 
-    dir_info = DirInfo.from_list(
-        [{"relpath": "1", "md5": "1"}, {"relpath": "2", "md5": "2"}]
-    )
-    with patch.object(
-        BaseTree, "_collect_dir", return_value=dir_info,
-    ):
-        hash1 = dvc.cache.local.tree.get_hash(path_info)
+    tree = Tree.from_list([{"relpath": "1", "md5": "1"}, {"relpath": "2", "md5": "2"}])
+    tree.digest()
+    with patch("dvc_data.hashfile.build._build_tree", return_value=(None, tree)):
+        _, _, obj = build(dvc.cache.local, path, dvc.cache.local.fs, "md5")
+        hash1 = obj.hash_info
 
-    dir_info = DirInfo.from_list(
-        [{"md5": "1", "relpath": "1"}, {"md5": "2", "relpath": "2"}]
-    )
-    with patch.object(
-        BaseTree, "_collect_dir", return_value=dir_info,
-    ):
-        hash2 = dvc.cache.local.tree.get_hash(path_info)
+    # remove the raw dir obj to force building the tree on the next build call
+    dvc.cache.local.fs.remove(dvc.cache.local.oid_to_path(hash1.as_raw().value))
+
+    tree = Tree.from_list([{"md5": "1", "relpath": "1"}, {"md5": "2", "relpath": "2"}])
+    tree.digest()
+    with patch("dvc_data.hashfile.build._build_tree", return_value=(None, tree)):
+        _, _, obj = build(dvc.cache.local, path, dvc.cache.local.fs, "md5")
+        hash2 = obj.hash_info
 
     assert hash1 == hash2
 
 
-def test_partial_push_n_pull(tmp_dir, dvc, tmp_path_factory, local_remote):
+def test_partial_push_n_pull(  # noqa: C901
+    tmp_dir, dvc, tmp_path_factory, local_remote
+):
+    from dvc_objects.fs import generic
+
     foo = tmp_dir.dvc_gen({"foo": "foo content"})[0].outs[0]
     bar = tmp_dir.dvc_gen({"bar": "bar content"})[0].outs[0]
-    baz = tmp_dir.dvc_gen({"baz": {"foo": "baz content"}})[0].outs[0]
+    baz = tmp_dir.dvc_gen({"baz": {"foo": "foo content"}})[0].outs[0]
 
     # Faulty upload version, failing on foo
-    original = LocalTree._upload
+    original = generic.transfer
+    odb = dvc.cloud.get_remote_odb("upstream")
 
-    def unreliable_upload(self, from_file, to_info, name=None, **kwargs):
-        if "foo" in name:
-            raise Exception("stop foo")
-        return original(self, from_file, to_info, name, **kwargs)
+    def unreliable_upload(from_fs, from_info, to_fs, to_info, **kwargs):
+        on_error = kwargs["on_error"]
+        assert on_error
+        if isinstance(from_info, str):
+            from_info = [from_info]
+        else:
+            from_info = list(from_info)
+        if isinstance(to_info, str):
+            to_info = [to_info]
+        else:
+            to_info = list(to_info)
+        for i in range(len(from_info) - 1, -1, -1):
+            from_i = from_info[i]
+            to_i = to_info[i]
+            if os.path.abspath(to_i) == os.path.abspath(
+                odb.get(foo.hash_info.value).path
+            ):
+                if on_error:
+                    on_error(from_i, to_i, Exception("stop foo"))
+                del from_info[i]
+                del to_info[i]
+        return original(from_fs, from_info, to_fs, to_info, **kwargs)
 
-    with patch.object(LocalTree, "_upload", unreliable_upload):
+    with patch.object(generic, "transfer", unreliable_upload):
         with pytest.raises(UploadError) as upload_error_info:
             dvc.push()
-        assert upload_error_info.value.amount == 3
+        assert upload_error_info.value.amount == 2
 
-        remote = dvc.cloud.get_remote("upstream")
-        assert not remote.tree.exists(
-            remote.tree.hash_to_path_info(foo.hash_info.value)
-        )
-        assert remote.tree.exists(
-            remote.tree.hash_to_path_info(bar.hash_info.value)
-        )
-        assert not remote.tree.exists(
-            remote.tree.hash_to_path_info(baz.hash_info.value)
-        )
+        assert not odb.exists(foo.hash_info.value)
+        assert odb.exists(bar.hash_info.value)
+        assert not odb.exists(baz.hash_info.value)
 
     # Push everything and delete local cache
     dvc.push()
-    remove(dvc.cache.local.cache_dir)
+    dvc.cache.local.clear()
 
-    with patch.object(LocalTree, "_download", side_effect=Exception):
+    baz._collect_used_dir_cache()
+
+    def unreliable_download(_from_fs, from_info, _to_fs, to_info, **kwargs):
+        on_error = kwargs["on_error"]
+        assert on_error
+        if isinstance(from_info, str):
+            from_info = [from_info]
+        if isinstance(to_info, str):
+            to_info = [to_info]
+        for from_i, to_i in zip(from_info, to_info):
+            on_error(from_i, to_i, Exception())
+
+    with patch.object(generic, "transfer", unreliable_download):
         with pytest.raises(DownloadError) as download_error_info:
             dvc.pull()
         # error count should be len(.dir + standalone file checksums)
         # since files inside dir are ignored if dir cache entry is missing
-        assert download_error_info.value.amount == 3
+        assert download_error_info.value.amount == 2
 
 
 def test_raise_on_too_many_open_files(
@@ -216,15 +228,14 @@ def test_raise_on_too_many_open_files(
 ):
     tmp_dir.dvc_gen({"file": "file content"})
 
-    mocker.patch.object(
-        LocalTree,
-        "_upload",
+    mocker.patch(
+        "dvc_objects.fs.generic.transfer",
         side_effect=OSError(errno.EMFILE, "Too many open files"),
     )
 
-    with pytest.raises(OSError) as e:
+    with pytest.raises(OSError, match="Too many open files") as e:
         dvc.push()
-        assert e.errno == errno.EMFILE
+    assert e.value.errno == errno.EMFILE
 
 
 def test_modify_missing_remote(tmp_dir, dvc):
@@ -233,12 +244,8 @@ def test_modify_missing_remote(tmp_dir, dvc):
 
 def test_remote_modify_local_on_repo_config(tmp_dir, dvc):
     assert main(["remote", "add", "myremote", "http://example.com/path"]) == 0
-    assert (
-        main(["remote", "modify", "myremote", "user", "xxx", "--local"]) == 0
-    )
-    assert dvc.config.load_one("local")["remote"]["myremote"] == {
-        "user": "xxx"
-    }
+    assert main(["remote", "modify", "myremote", "user", "xxx", "--local"]) == 0
+    assert dvc.config.load_one("local")["remote"]["myremote"] == {"user": "xxx"}
     assert dvc.config.load_one("repo")["remote"]["myremote"] == {
         "url": "http://example.com/path"
     }
@@ -246,6 +253,7 @@ def test_remote_modify_local_on_repo_config(tmp_dir, dvc):
     assert dvc.config["remote"]["myremote"] == {
         "url": "http://example.com/path",
         "user": "xxx",
+        "verify": False,
     }
 
 
@@ -253,45 +261,45 @@ def test_external_dir_resource_on_no_cache(tmp_dir, dvc, tmp_path_factory):
     # https://github.com/iterative/dvc/issues/2647, is some situations
     # (external dir dependency) cache is required to calculate dir md5
     external_dir = tmp_path_factory.mktemp("external_dir")
-    (external_dir / "file").write_text("content")
+    file = external_dir / "file"
 
     dvc.cache.local = None
     with pytest.raises(RemoteCacheRequiredError):
         dvc.run(
-            cmd="echo hello world",
-            outs=[os.fspath(external_dir)],
-            single_stage=True,
+            cmd=f"echo content > {file}",
+            outs=[os.fspath(file)],
+            name="echo",
+            external=True,
         )
 
 
 def test_push_order(tmp_dir, dvc, tmp_path_factory, mocker, local_remote):
+    from dvc_objects.fs import generic
+
     foo = tmp_dir.dvc_gen({"foo": {"bar": "bar content"}})[0].outs[0]
     tmp_dir.dvc_gen({"baz": "baz content"})
 
-    mocked_upload = mocker.patch.object(LocalTree, "_upload", return_value=0)
+    mocked_upload = mocker.spy(generic, "transfer")
     dvc.push()
 
     # foo .dir file should be uploaded after bar
-    remote = dvc.cloud.get_remote("upstream")
-    foo_path = remote.tree.hash_to_path_info(foo.hash_info.value)
-    bar_path = remote.tree.hash_to_path_info(
-        foo.hash_info.dir_info.trie[("bar",)].value
+    odb = dvc.cloud.get_remote_odb("upstream")
+    foo_path = odb.oid_to_path(foo.hash_info.value)
+    bar_path = odb.oid_to_path(foo.obj._trie[("bar",)][1].value)
+    paths = list(
+        itertools.chain.from_iterable(
+            args[3] for args, _ in mocked_upload.call_args_list
+        )
     )
-    paths = [args[1] for args, _ in mocked_upload.call_args_list]
     assert paths.index(foo_path) > paths.index(bar_path)
 
 
 def test_remote_modify_validation(dvc):
     remote_name = "drive"
     unsupported_config = "unsupported_config"
+    assert main(["remote", "add", "-d", remote_name, "gdrive://test/test"]) == 0
     assert (
-        main(["remote", "add", "-d", remote_name, "gdrive://test/test"]) == 0
-    )
-    assert (
-        main(
-            ["remote", "modify", remote_name, unsupported_config, "something"]
-        )
-        == 251
+        main(["remote", "modify", remote_name, unsupported_config, "something"]) == 251
     )
     config = configobj.ConfigObj(dvc.config.files["repo"])
     assert unsupported_config not in config[f'remote "{remote_name}"']
@@ -302,20 +310,14 @@ def test_remote_modify_unset(dvc):
     config = configobj.ConfigObj(dvc.config.files["repo"])
     assert config['remote "myremote"'] == {"url": "gdrive://test/test"}
 
-    assert (
-        main(["remote", "modify", "myremote", "gdrive_client_id", "something"])
-        == 0
-    )
+    assert main(["remote", "modify", "myremote", "gdrive_client_id", "something"]) == 0
     config = configobj.ConfigObj(dvc.config.files["repo"])
     assert config['remote "myremote"'] == {
         "url": "gdrive://test/test",
         "gdrive_client_id": "something",
     }
 
-    assert (
-        main(["remote", "modify", "myremote", "gdrive_client_id", "--unset"])
-        == 0
-    )
+    assert main(["remote", "modify", "myremote", "gdrive_client_id", "--unset"]) == 0
     config = configobj.ConfigObj(dvc.config.files["repo"])
     assert config['remote "myremote"'] == {"url": "gdrive://test/test"}
 
@@ -413,10 +415,8 @@ def test_protect_local_remote(tmp_dir, dvc, local_remote):
     (stage,) = tmp_dir.dvc_gen("file", "file content")
 
     dvc.push()
-    remote = dvc.cloud.get_remote("upstream")
-    remote_cache_file = remote.tree.hash_to_path_info(
-        stage.outs[0].hash_info.value
-    )
+    odb = dvc.cloud.get_remote_odb("upstream")
+    remote_cache_file = odb.oid_to_path(stage.outs[0].hash_info.value)
 
     assert os.path.exists(remote_cache_file)
     assert stat.S_IMODE(os.stat(remote_cache_file).st_mode) == 0o444
@@ -424,37 +424,16 @@ def test_protect_local_remote(tmp_dir, dvc, local_remote):
 
 def test_push_incomplete_dir(tmp_dir, dvc, mocker, local_remote):
     (stage,) = tmp_dir.dvc_gen({"dir": {"foo": "foo", "bar": "bar"}})
-    remote = dvc.cloud.get_remote("upstream")
+    remote_odb = dvc.cloud.get_remote_odb("upstream")
 
-    cache = dvc.cache.local
-    dir_hash = stage.outs[0].hash_info.value
-    used = stage.get_used_cache(remote=remote)
+    odb = dvc.cache.local
+    out = stage.outs[0]
+    file_objs = [entry_obj for _, _, entry_obj in out.obj]
 
     # remove one of the cache files for directory
-    file_hashes = list(used.child_keys(cache.tree.scheme, dir_hash))
-    remove(cache.tree.hash_to_path_info(file_hashes[0]))
+    remove(odb.oid_to_path(file_objs[0].value))
 
     dvc.push()
-    assert not remote.tree.exists(remote.tree.hash_to_path_info(dir_hash))
-    assert not remote.tree.exists(
-        remote.tree.hash_to_path_info(file_hashes[0])
-    )
-    assert remote.tree.exists(remote.tree.hash_to_path_info(file_hashes[1]))
-
-
-def test_upload_exists(tmp_dir, dvc, local_remote):
-    tmp_dir.gen("foo", "foo")
-    remote = dvc.cloud.get_remote("upstream")
-    # allow uploaded files to be writable for this test,
-    # normally they are set to read-only for DVC remotes
-    remote.tree.CACHE_MODE = 0o644
-
-    from_info = PathInfo(tmp_dir / "foo")
-    to_info = remote.tree.path_info / "foo"
-    remote.tree.upload(from_info, to_info)
-    assert remote.tree.exists(to_info)
-
-    tmp_dir.gen("foo", "bar")
-    remote.tree.upload(from_info, to_info)
-    with remote.tree.open(to_info) as fobj:
-        assert fobj.read() == "bar"
+    assert not remote_odb.exists(out.hash_info.value)
+    assert not remote_odb.exists(file_objs[0].value)
+    assert remote_odb.exists(file_objs[1].value)
