@@ -1,11 +1,11 @@
-from dvc.cache import Cache
-from dvc.dvcfile import PIPELINE_FILE, PIPELINE_LOCK
-from dvc.system import System
+from dvc.cachemgr import CacheManager
+from dvc.dvcfile import LOCK_FILE, PROJECT_FILE
+from dvc.fs import system
 
 
 def test_destroy(tmp_dir, dvc, run_copy):
     dvc.config["cache"]["type"] = ["symlink"]
-    dvc.cache = Cache(dvc)
+    dvc.cache = CacheManager(dvc)
 
     tmp_dir.dvc_gen("file", "text")
     tmp_dir.dvc_gen({"dir": {"file": "lorem", "subdir/file": "ipsum"}})
@@ -19,11 +19,12 @@ def test_destroy(tmp_dir, dvc, run_copy):
     # Remove all the files related to DVC
     for path in [
         ".dvc",
+        ".dvcignore",
         "file.dvc",
         "file2.dvc",
         "dir.dvc",
-        PIPELINE_FILE,
-        PIPELINE_LOCK,
+        PROJECT_FILE,
+        LOCK_FILE,
     ]:
         assert not (tmp_dir / path).exists()
 
@@ -49,4 +50,4 @@ def test_destroy(tmp_dir, dvc, run_copy):
         "dir/subdir",
         "dir/subdir/file",
     ]:
-        assert not System.is_symlink(tmp_dir / path)
+        assert not system.is_symlink(tmp_dir / path)
