@@ -1,5 +1,6 @@
 from dvc.cli import parse_args
-from dvc.command.run import CmdRun
+from dvc.commands.run import CmdRun
+from tests.utils.asserts import called_once_with_subset
 
 
 def test_run(mocker, dvc):
@@ -7,7 +8,7 @@ def test_run(mocker, dvc):
         [
             "run",
             "--name",
-            "nam",
+            "name",
             "--deps",
             "deps",
             "--outs",
@@ -54,7 +55,8 @@ def test_run(mocker, dvc):
 
     assert cmd.run() == 0
 
-    m.assert_called_once_with(
+    assert called_once_with_subset(
+        m,
         deps=["deps"],
         outs=["outs"],
         outs_no_cache=["outs-no-cache"],
@@ -74,7 +76,7 @@ def test_run(mocker, dvc):
         no_commit=True,
         always_changed=True,
         cmd="command",
-        name="nam",
+        name="name",
         single_stage=False,
         external=True,
         desc="description",
@@ -86,7 +88,8 @@ def test_run_args_from_cli(mocker, dvc):
     cmd = args.func(args)
     m = mocker.patch.object(cmd.repo, "run", autospec=True)
     assert cmd.run() == 0
-    m.assert_called_once_with(
+    assert called_once_with_subset(
+        m,
         deps=[],
         outs=[],
         outs_no_cache=[],
@@ -118,7 +121,8 @@ def test_run_args_with_spaces(mocker, dvc):
     cmd = args.func(args)
     m = mocker.patch.object(cmd.repo, "run", autospec=True)
     assert cmd.run() == 0
-    m.assert_called_once_with(
+    assert called_once_with_subset(
+        m,
         deps=[],
         outs=[],
         outs_no_cache=[],
