@@ -1,17 +1,19 @@
 import os
+from unittest.mock import MagicMock, mock_open, patch
 
 import pytest
-from mock import MagicMock, mock_open, patch
 
 from dvc.ignore import DvcIgnorePatterns
 
 
 def mock_dvcignore(dvcignore_path, patterns):
-    tree = MagicMock()
-    with patch.object(tree, "open", mock_open(read_data="\n".join(patterns))):
-        ignore_patterns = DvcIgnorePatterns.from_files(dvcignore_path, tree)
+    from dvc.fs import localfs
 
-    return ignore_patterns
+    fs = MagicMock()
+    fs.path = localfs.path
+    fs.sep = localfs.sep
+    with patch.object(fs, "open", mock_open(read_data="\n".join(patterns))):
+        return DvcIgnorePatterns.from_file(dvcignore_path, fs, "mocked")
 
 
 @pytest.mark.parametrize(
@@ -39,7 +41,7 @@ def mock_dvcignore(dvcignore_path, patterns):
         ("to_ignore.txt", ["!to_ignore.txt", "to_ignore*"], True),
         # It is not possible to re-include a file if a parent directory of
         # that file is excluded.
-        # Git doesn’t list excluded directories for performance reasons,
+        # Git doesn't list excluded directories for performance reasons,
         # so any patterns on contained files have no effect,
         # no matter where they are defined.
         # see (`tests/func/test_ignore.py::test_ignore_parent_path`)
@@ -85,7 +87,7 @@ def mock_dvcignore(dvcignore_path, patterns):
         ("to_ignore.txt", ["/*.txt"], True),
         (os.path.join("path", "to_ignore.txt"), ["/*.txt"], False),
         (os.path.join("data", "file.txt"), ["data/*"], True),
-        (os.path.join("data", "subdir", "file.txt"), ["data/*"], False),
+        # (os.path.join("data", "subdir", "file.txt"), ["data/*"], False),
         (os.path.join("data", "file.txt"), ["data/"], True),
         (os.path.join("data", "subdir", "file.txt"), ["data/"], True),
         (os.path.join("data", "subdir", "file.txt"), ["subdir/"], True),
@@ -116,7 +118,7 @@ def mock_dvcignore(dvcignore_path, patterns):
         # same as pattern "foo".
         # "**/foo/bar" matches file or directory "bar" anywhere that is
         # directly under directory "foo".
-        (os.path.join("rel", "p", "p2", "to_ignore"), ["**/to_ignore"], True,),
+        (os.path.join("rel", "p", "p2", "to_ignore"), ["**/to_ignore"], True),
         (
             os.path.join("rel", "p", "p2", "to_ignore"),
             ["**/p2/to_ignore"],
@@ -130,8 +132,8 @@ def mock_dvcignore(dvcignore_path, patterns):
         # A trailing "/**" matches everything inside.
         # For example, "abc/**" matches all files inside directory "abc",
         # relative to the location of the .gitignore file, with infinite depth.
-        (os.path.join("rel", "p", "p2", "to_ignore"), ["rel/**"], True,),
-        (os.path.join("rel", "p", "p2", "to_ignore"), ["p/**"], False,),
+        (os.path.join("rel", "p", "p2", "to_ignore"), ["rel/**"], True),
+        (os.path.join("rel", "p", "p2", "to_ignore"), ["p/**"], False),
         (
             os.path.join("rel", "path", "path2", "dont_ignore"),
             ["rel/**"],
@@ -140,7 +142,7 @@ def mock_dvcignore(dvcignore_path, patterns):
         # A slash followed by two consecutive asterisks then a slash matches
         # zero or more directories.
         # For example, "a/**/b" matches "a/b", "a/x/b", "a/x/y/b" and so on.
-        (os.path.join("rel", "p", "to_ignore"), ["rel/**/to_ignore"], True,),
+        (os.path.join("rel", "p", "to_ignore"), ["rel/**/to_ignore"], True),
         (
             os.path.join("rel", "p", "p2", "to_ignore"),
             ["rel/**/to_ignore"],
@@ -172,10 +174,7 @@ def mock_dvcignore(dvcignore_path, patterns):
         ),
     ],
 )
-def test_match_ignore_from_file(
-    file_to_ignore_relpath, patterns, expected_match
-):
-
+def test_match_ignore_from_file(file_to_ignore_relpath, patterns, expected_match):
     dvcignore_path = os.path.join(
         os.path.sep, "full", "path", "to", "ignore", "file", ".dvcignore"
     )
@@ -184,8 +183,7 @@ def test_match_ignore_from_file(
     ignore_file = mock_dvcignore(dvcignore_path, patterns)
 
     assert (
-        ignore_file.matches(dvcignore_dirname, file_to_ignore_relpath)
-        == expected_match
+        ignore_file.matches(dvcignore_dirname, file_to_ignore_relpath) == expected_match
     )
 
 
@@ -193,7 +191,7 @@ def test_match_ignore_from_file(
 @pytest.mark.parametrize("omit_dir", [".git", ".hg", ".dvc"])
 def test_should_ignore_dir(omit_dir, sub_dir):
     root = os.path.join(os.path.sep, "walk", "dir", "root")
-    ignore = DvcIgnorePatterns([".git/", ".hg/", ".dvc/"], root)
+    ignore = DvcIgnorePatterns([".git/", ".hg/", ".dvc/"], root, os.sep)
 
     dirs = [omit_dir, "dir1", "dir2"]
     files = [omit_dir, "file1", "file2"]
