@@ -1,8 +1,7 @@
 import logging
+
 from dvc.progress import Tqdm
 from dvc.utils import env2bool
-import sys
-import mock
 
 
 def test_quiet_logging(caplog, capsys):
@@ -14,6 +13,17 @@ def test_quiet_logging(caplog, capsys):
         assert out_err.err == ""
 
 
+def test_quiet_logging_disable_false(caplog, capsys, mocker):
+    # simulate interactive terminal
+    mocker.patch("sys.stdout.isatty", return_value=True)
+    with caplog.at_level(logging.CRITICAL, logger="dvc"):
+        for _ in Tqdm(range(10), disable=False):
+            pass
+        out_err = capsys.readouterr()
+        assert out_err.out == ""
+        assert out_err.err == ""
+
+
 def test_quiet_notty(caplog, capsys):
     with caplog.at_level(logging.INFO, logger="dvc"):
         for _ in Tqdm(range(10)):
@@ -26,12 +36,13 @@ def test_quiet_notty(caplog, capsys):
             assert out_err.err == ""
 
 
-def test_default(caplog, capsys):
+def test_default(caplog, capsys, mocker):
+    # simulate interactive terminal
+    mocker.patch("sys.stdout.isatty", return_value=True)
     with caplog.at_level(logging.INFO, logger="dvc"):
-        # simulate interactive terminal
-        with mock.patch.object(sys.stderr, "isatty", return_value=True):
-            for _ in Tqdm(range(10)):
-                pass
-            out_err = capsys.readouterr()
-            assert out_err.out == ""
-            assert "0/10" in out_err.err
+        for _ in Tqdm(range(10)):
+            pass
+
+        out_err = capsys.readouterr()
+        assert out_err.out == ""
+        assert "0/10" in out_err.err
