@@ -1,46 +1,8 @@
 import os
-import re
 
 import pytest
 
-from dvc.path_info import PathInfo
-from dvc.utils import (
-    dict_sha256,
-    file_md5,
-    fix_env,
-    parse_target,
-    relpath,
-    resolve_output,
-    tmp_fname,
-    to_chunks,
-)
-
-
-@pytest.mark.parametrize(
-    "chunk_size, expected_chunks",
-    [(1, [[1], [2], [3], [4]]), (2, [[1, 2], [3, 4]]), (3, [[1, 2, 3], [4]])],
-)
-def test_to_chunks_chunk_size(chunk_size, expected_chunks):
-    list_to_chunk = [1, 2, 3, 4]
-    result = list(to_chunks(list_to_chunk, chunk_size=chunk_size))
-    assert result == expected_chunks
-
-
-@pytest.mark.parametrize("num_chunks, chunk_size", [(1, 2), (None, None)])
-def test_to_chunks_should_raise(num_chunks, chunk_size):
-    list_to_chunk = [1, 2, 3]
-    with pytest.raises(ValueError):
-        to_chunks(list_to_chunk, num_chunks, chunk_size)
-
-
-@pytest.mark.parametrize(
-    "num_chunks, expected_chunks",
-    [(4, [[1], [2], [3], [4]]), (3, [[1, 2], [3, 4]]), (2, [[1, 2], [3, 4]])],
-)
-def test_to_chunks_num_chunks(num_chunks, expected_chunks):
-    list_to_chunk = [1, 2, 3, 4]
-    result = to_chunks(list_to_chunk, num_chunks=num_chunks)
-    assert result == expected_chunks
+from dvc.utils import dict_sha256, fix_env, parse_target, relpath, resolve_output
 
 
 @pytest.mark.skipif(os.name == "nt", reason="pyenv-win is not supported")
@@ -48,10 +10,7 @@ def test_to_chunks_num_chunks(num_chunks, expected_chunks):
     "path, orig",
     [
         (
-            (
-                "/pyenv/bin:/pyenv/libexec:/pyenv/plugins/plugin:"
-                "/orig/path1:/orig/path2"
-            ),
+            "/pyenv/bin:/pyenv/libexec:/pyenv/plugins/plugin:/orig/path1:/orig/path2",
             "/orig/path1:/orig/path2",
         ),
         (
@@ -59,10 +18,7 @@ def test_to_chunks_num_chunks(num_chunks, expected_chunks):
             "/orig/path1:/orig/path2",
         ),
         (
-            (
-                "/pyenv/bin:/some/libexec:/pyenv/plugins/plugin:"
-                "/orig/path1:/orig/path2"
-            ),
+            "/pyenv/bin:/some/libexec:/pyenv/plugins/plugin:/orig/path1:/orig/path2",
             "/orig/path1:/orig/path2",
         ),
         ("/orig/path1:/orig/path2", "/orig/path1:/orig/path2"),
@@ -83,32 +39,25 @@ def test_fix_env_pyenv(path, orig):
     assert fix_env(env)["PATH"] == orig
 
 
-def test_file_md5(tmp_dir):
-    tmp_dir.gen("foo", "foo content")
+@pytest.mark.skipif(os.name != "nt", reason="Windows specific")
+def test_relpath_windows(monkeypatch):
+    """test that relpath correctly generated when run on a
+    windows network share. The drive mapped path is mapped
+    to a UNC path by os.path.realpath"""
 
-    assert file_md5("foo") == file_md5(PathInfo("foo"))
+    def dummy_realpath(path):
+        return path.replace("x:", "\\\\server\\share")
 
-
-def test_tmp_fname():
-    file_path = os.path.join("path", "to", "file")
-    file_path_info = PathInfo(file_path)
-
-    def pattern(path):
-        return r"^" + re.escape(path) + r"\.[a-z0-9]{22}\.tmp$"
-
-    assert re.search(pattern(file_path), tmp_fname(file_path), re.IGNORECASE)
-    assert re.search(
-        pattern(file_path_info.fspath),
-        tmp_fname(file_path_info),
-        re.IGNORECASE,
+    monkeypatch.setattr(os.path, "realpath", dummy_realpath)
+    assert (
+        relpath("x:\\dir1\\dir2\\file.txt", "\\\\server\\share\\dir1")
+        == "dir2\\file.txt"
     )
 
-
-def test_relpath():
-    path = "path"
-    path_info = PathInfo(path)
-
-    assert relpath(path) == relpath(path_info)
+    assert (
+        relpath("y:\\dir1\\dir2\\file.txt", "\\\\server\\share\\dir1")
+        == "y:\\dir1\\dir2\\file.txt"
+    )
 
 
 @pytest.mark.parametrize(
@@ -181,9 +130,9 @@ def test_parse_target(inp, out, default):
 
 
 def test_hint_on_lockfile():
-    with pytest.raises(Exception) as exc:
+    with pytest.raises(Exception, match="Did you mean: `dvc.yaml:name`?") as e:
         assert parse_target("dvc.lock:name")
-    assert "dvc.yaml:name" in str(exc.value)
+    assert "dvc.yaml:name" in str(e.value)
 
 
 @pytest.mark.parametrize(
